/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.wso2.registry.Tag;
import org.wso2.registry.TaggedResourcePath;
import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.registry.jdbc.dataobjects.TagDO;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class TagsDAO {

    public void addTag(String tag, Connection conn) throws SQLException {

        String sql = "INSERT INTO TAGS (TAG_NAME) VALUES (?)";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        s.executeUpdate();
    }

    public TagDO getTag(String tag, Connection conn) throws SQLException {

        TagDO tagDO = null;

        String sql = "SELECT * FROM TAGS T WHERE T.TAG_NAME=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();

        if (result.next()) {
            tagDO = new TagDO();
            tagDO.setId(result.getLong(DatabaseConstants.TAG_ID_FIELD));
            tagDO.setTagName(result.getString(DatabaseConstants.TAG_NAME_FIELD));
        }

        return tagDO;
    }

    public boolean tagExist(String tag, Connection conn) throws SQLException {

        String sql = "SELECT COUNT(T.TAG_NAME) FROM TAGS T WHERE T.TAG_NAME=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();
        long count = 0;
        if (result.next()) {
            count = result.getLong(1);
        }

        return count == 1;
    }

    public void addTagging(long tagID, long artifactID, String userID, Connection conn) throws SQLException {

        String sql = "INSERT INTO TAGGINGS (TAG_ID, AID, USER_ID, TAGGED_TIME) VALUES (?,?,?,?)";

        long now = System.currentTimeMillis();

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, tagID);
        s.setLong(2, artifactID);
        s.setString(3, userID);
        s.setDate(4, new Date(now));

        s.executeUpdate();
    }

    public boolean taggingExist(long tagID, long artifactID, String userID, Connection conn)
            throws SQLException {

        String sql = "SELECT COUNT(TN.TAG_ID) FROM TAGGINGS TN WHERE TN.TAG_ID=? AND TN.AID=? AND TN.USER_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, tagID);
        s.setLong(2, artifactID);
        s.setString(3, userID);

        ResultSet result = s.executeQuery();
        long count = 0;
        if (result.next()) {
            count = result.getLong(1);
        }

        return count == 1? true: false;                
    }

    public void removeTaggings(long artifactID, Connection conn) throws SQLException {

        String sql = "DELETE FROM TAGGINGS WHERE AID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);

        s.executeUpdate();
    }

    public String[] getPathsWithTag(String tag, Connection conn) throws SQLException {

        String sql = "SELECT A.PATH FROM ARTIFACTS A, TAGGINGS TN, TAGS T WHERE T.TAG_NAME=? AND T.TAG_ID=TN.TAG_ID AND TN.AID=A.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();
        List pathList = new ArrayList();
        while (result.next()) {
            pathList.add(result.getString(DatabaseConstants.PATH_FIELD));
        }

        String[] paths = (String[]) pathList.toArray(new String[pathList.size()]);        
        return paths;
    }    

    public String[] getTags(String path, Connection conn) throws SQLException {

        String sql = "SELECT T.TAG_NAME FROM TAGS T, TAGGINGS TN, ARTIFACTS A WHERE A.PATH=? AND A.AID=TN.AID AND TN.TAG_ID=T.TAG_ID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);

        ResultSet result = s.executeQuery();
        List tagList = new ArrayList();
        while (result.next()) {
            tagList.add(result.getString(DatabaseConstants.TAG_NAME_FIELD));
        }

        String[] tags = (String[]) tagList.toArray(new String[tagList.size()]);
        return tags;
    }

    public List getTaggedPaths(String tag, Connection conn) throws SQLException {

        String sql = "SELECT A.PATH, COUNT(T.TAG_ID) FROM ARTIFACTS A, TAGGINGS TN, TAGS T WHERE T.TAG_NAME=? AND T.TAG_ID=TN.TAG_ID AND TN.AID=A.AID GROUP BY A.PATH";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, tag);

        ResultSet result = s.executeQuery();
        List pathList = new ArrayList();
        while (result.next()) {
            TaggedResourcePath taggedResourcePath = new TaggedResourcePath();
            taggedResourcePath.setResourcePath(result.getString(DatabaseConstants.PATH_FIELD));
            taggedResourcePath.setTagCount(result.getLong(2));
            pathList.add(taggedResourcePath);
        }

        return pathList;
    }

    public Tag[] getTagsWithCount(String path, Connection conn) throws SQLException {

        String sql = "SELECT T.TAG_NAME, COUNT(A.AID) FROM TAGS T, TAGGINGS TN, ARTIFACTS A WHERE A.PATH=? AND A.AID=TN.AID AND TN.TAG_ID=T.TAG_ID GROUP BY T.TAG_NAME";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);

        ResultSet result = s.executeQuery();
        List tagList = new ArrayList();
        while (result.next()) {
            Tag tag = new Tag();
            tag.setTagName(result.getString(DatabaseConstants.TAG_NAME_FIELD));
            tag.setTagCount(result.getLong(2));
            tagList.add(tag);
        }

        Tag[] tags = (Tag[]) tagList.toArray(new Tag[tagList.size()]);
        return tags;
    }
}
